"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstance = exports.ActiveWorkflowRunner = void 0;
const _1 = require("./");
const n8n_core_1 = require("n8n-core");
const n8n_workflow_1 = require("n8n-workflow");
class ActiveWorkflowRunner {
    constructor() {
        this.activeWorkflows = null;
        this.activationErrors = {};
    }
    async init() {
        var _a;
        const workflowsData = await _1.Db.collections.Workflow.find({ active: true });
        await ((_a = _1.Db.collections.Webhook) === null || _a === void 0 ? void 0 : _a.clear());
        this.activeWorkflows = new n8n_core_1.ActiveWorkflows();
        if (workflowsData.length !== 0) {
            console.log('\n ================================');
            console.log('   Start Active Workflows:');
            console.log(' ================================');
            for (const workflowData of workflowsData) {
                console.log(`   - ${workflowData.name}`);
                try {
                    await this.add(workflowData.id.toString(), workflowData);
                    console.log(`     => Started`);
                }
                catch (error) {
                    console.log(`     => ERROR: Workflow could not be activated:`);
                    console.log(`               ${error.message}`);
                }
            }
        }
    }
    async removeAll() {
        const activeWorkflowId = [];
        if (this.activeWorkflows !== null) {
            activeWorkflowId.push.apply(activeWorkflowId, this.activeWorkflows.allActiveWorkflows());
        }
        const activeWorkflows = await this.getActiveWorkflows();
        activeWorkflowId.push.apply(activeWorkflowId, activeWorkflows.map(workflow => workflow.id));
        const removePromises = [];
        for (const workflowId of activeWorkflowId) {
            removePromises.push(this.remove(workflowId));
        }
        await Promise.all(removePromises);
        return;
    }
    async executeWebhook(httpMethod, path, req, res) {
        var _a, _b;
        if (this.activeWorkflows === null) {
            throw new _1.ResponseHelper.ResponseError('The "activeWorkflows" instance did not get initialized yet.', 404, 404);
        }
        req.params = {};
        let webhook = await ((_a = _1.Db.collections.Webhook) === null || _a === void 0 ? void 0 : _a.findOne({ webhookPath: path, method: httpMethod }));
        let webhookId;
        if (webhook === undefined) {
            const pathElements = path.split('/');
            webhookId = pathElements.shift();
            const dynamicWebhooks = await ((_b = _1.Db.collections.Webhook) === null || _b === void 0 ? void 0 : _b.find({ webhookId, method: httpMethod, pathLength: pathElements.length }));
            if (dynamicWebhooks === undefined || dynamicWebhooks.length === 0) {
                throw new _1.ResponseHelper.ResponseError(`The requested webhook "${httpMethod} ${path}" is not registered.`, 404, 404);
            }
            webhook = dynamicWebhooks[0];
            if (dynamicWebhooks.length > 1) {
                let maxMatches = 0;
                const pathElementsSet = new Set(pathElements);
                dynamicWebhooks.forEach(dynamicWebhook => {
                    const intersection = dynamicWebhook.webhookPath
                        .split('/')
                        .reduce((acc, element) => pathElementsSet.has(element) ? acc += 1 : acc, 0);
                    if (intersection > maxMatches) {
                        maxMatches = intersection;
                        webhook = dynamicWebhook;
                    }
                });
                if (maxMatches === 0) {
                    throw new _1.ResponseHelper.ResponseError(`The requested webhook "${httpMethod} ${path}" is not registered.`, 404, 404);
                }
            }
            path = webhook.webhookPath;
            webhook.webhookPath.split('/').forEach((ele, index) => {
                if (ele.startsWith(':')) {
                    req.params[ele.slice(1)] = pathElements[index];
                }
            });
        }
        const workflowData = await _1.Db.collections.Workflow.findOne(webhook.workflowId);
        if (workflowData === undefined) {
            throw new _1.ResponseHelper.ResponseError(`Could not find workflow with id "${webhook.workflowId}"`, 404, 404);
        }
        const nodeTypes = _1.NodeTypes();
        const workflow = new n8n_workflow_1.Workflow({ id: webhook.workflowId.toString(), name: workflowData.name, nodes: workflowData.nodes, connections: workflowData.connections, active: workflowData.active, nodeTypes, staticData: workflowData.staticData, settings: workflowData.settings });
        const credentials = await _1.WorkflowCredentials([workflow.getNode(webhook.node)]);
        const additionalData = await _1.WorkflowExecuteAdditionalData.getBase(credentials);
        const webhookData = n8n_workflow_1.NodeHelpers.getNodeWebhooks(workflow, workflow.getNode(webhook.node), additionalData).filter((webhook) => {
            return (webhook.httpMethod === httpMethod && webhook.path === path);
        })[0];
        const workflowStartNode = workflow.getNode(webhookData.node);
        if (workflowStartNode === null) {
            throw new _1.ResponseHelper.ResponseError('Could not find node to process webhook.', 404, 404);
        }
        return new Promise((resolve, reject) => {
            const executionMode = 'webhook';
            _1.WebhookHelpers.executeWebhook(workflow, webhookData, workflowData, workflowStartNode, executionMode, undefined, req, res, (error, data) => {
                if (error !== null) {
                    return reject(error);
                }
                resolve(data);
            });
        });
    }
    async getWebhookMethods(path) {
        var _a;
        const webhooks = await ((_a = _1.Db.collections.Webhook) === null || _a === void 0 ? void 0 : _a.find({ webhookPath: path }));
        const webhookMethods = webhooks.map(webhook => webhook.method);
        return webhookMethods;
    }
    async getActiveWorkflows() {
        var _a;
        const activeWorkflows = await ((_a = _1.Db.collections.Workflow) === null || _a === void 0 ? void 0 : _a.find({ where: { active: true }, select: ['id'] }));
        return activeWorkflows.filter(workflow => this.activationErrors[workflow.id.toString()] === undefined);
    }
    async isActive(id) {
        var _a;
        const workflow = await ((_a = _1.Db.collections.Workflow) === null || _a === void 0 ? void 0 : _a.findOne({ id }));
        return workflow === null || workflow === void 0 ? void 0 : workflow.active;
    }
    getActivationError(id) {
        if (this.activationErrors[id] === undefined) {
            return undefined;
        }
        return this.activationErrors[id];
    }
    async addWorkflowWebhooks(workflow, additionalData, mode) {
        var _a;
        const webhooks = _1.WebhookHelpers.getWorkflowWebhooks(workflow, additionalData);
        let path = '';
        for (const webhookData of webhooks) {
            const node = workflow.getNode(webhookData.node);
            node.name = webhookData.node;
            path = node.parameters.path;
            if (node.parameters.path === undefined) {
                path = workflow.expression.getSimpleParameterValue(node, webhookData.webhookDescription['path'], mode);
                if (path === undefined) {
                    console.error(`No webhook path could be found for node "${node.name}" in workflow "${workflow.id}".`);
                    continue;
                }
            }
            const isFullPath = workflow.expression.getSimpleParameterValue(node, webhookData.webhookDescription['isFullPath'], mode, false);
            const webhook = {
                workflowId: webhookData.workflowId,
                webhookPath: n8n_workflow_1.NodeHelpers.getNodeWebhookPath(workflow.id, node, path, isFullPath),
                node: node.name,
                method: webhookData.httpMethod,
            };
            if (webhook.webhookPath.startsWith('/')) {
                webhook.webhookPath = webhook.webhookPath.slice(1);
            }
            if ((path.startsWith(':') || path.includes('/:')) && node.webhookId) {
                webhook.webhookId = node.webhookId;
                webhook.pathLength = webhook.webhookPath.split('/').length;
            }
            try {
                await ((_a = _1.Db.collections.Webhook) === null || _a === void 0 ? void 0 : _a.insert(webhook));
                const webhookExists = await workflow.runWebhookMethod('checkExists', webhookData, n8n_core_1.NodeExecuteFunctions, mode, false);
                if (webhookExists !== true) {
                    await workflow.runWebhookMethod('create', webhookData, n8n_core_1.NodeExecuteFunctions, mode, false);
                }
            }
            catch (error) {
                try {
                    await this.removeWorkflowWebhooks(workflow.id);
                }
                catch (error) {
                    console.error(`Could not remove webhooks of workflow "${workflow.id}" because of error: "${error.message}"`);
                }
                let errorMessage = '';
                if (error.name === 'QueryFailedError') {
                    errorMessage = `The webhook path [${webhook.webhookPath}] and method [${webhook.method}] already exist.`;
                }
                else if (error.detail) {
                    errorMessage = error.detail;
                }
                else {
                    errorMessage = error.message;
                }
                throw new Error(errorMessage);
            }
        }
        await _1.WorkflowHelpers.saveStaticData(workflow);
    }
    async removeWorkflowWebhooks(workflowId) {
        var _a;
        const workflowData = await _1.Db.collections.Workflow.findOne(workflowId);
        if (workflowData === undefined) {
            throw new Error(`Could not find workflow with id "${workflowId}"`);
        }
        const nodeTypes = _1.NodeTypes();
        const workflow = new n8n_workflow_1.Workflow({ id: workflowId, name: workflowData.name, nodes: workflowData.nodes, connections: workflowData.connections, active: workflowData.active, nodeTypes, staticData: workflowData.staticData, settings: workflowData.settings });
        const mode = 'internal';
        const credentials = await _1.WorkflowCredentials(workflowData.nodes);
        const additionalData = await _1.WorkflowExecuteAdditionalData.getBase(credentials);
        const webhooks = _1.WebhookHelpers.getWorkflowWebhooks(workflow, additionalData);
        for (const webhookData of webhooks) {
            await workflow.runWebhookMethod('delete', webhookData, n8n_core_1.NodeExecuteFunctions, mode, false);
        }
        await _1.WorkflowHelpers.saveStaticData(workflow);
        const webhook = {
            workflowId: workflowData.id,
        };
        await ((_a = _1.Db.collections.Webhook) === null || _a === void 0 ? void 0 : _a.delete(webhook));
    }
    runWorkflow(workflowData, node, data, additionalData, mode) {
        const nodeExecutionStack = [
            {
                node,
                data: {
                    main: data,
                },
            },
        ];
        const executionData = {
            startData: {},
            resultData: {
                runData: {},
            },
            executionData: {
                contextData: {},
                nodeExecutionStack,
                waitingExecution: {},
            },
        };
        const runData = {
            credentials: additionalData.credentials,
            executionMode: mode,
            executionData,
            workflowData,
        };
        const workflowRunner = new _1.WorkflowRunner();
        return workflowRunner.run(runData, true);
    }
    getExecutePollFunctions(workflowData, additionalData, mode) {
        return ((workflow, node) => {
            const returnFunctions = n8n_core_1.NodeExecuteFunctions.getExecutePollFunctions(workflow, node, additionalData, mode);
            returnFunctions.__emit = (data) => {
                this.runWorkflow(workflowData, node, data, additionalData, mode);
            };
            return returnFunctions;
        });
    }
    getExecuteTriggerFunctions(workflowData, additionalData, mode) {
        return ((workflow, node) => {
            const returnFunctions = n8n_core_1.NodeExecuteFunctions.getExecuteTriggerFunctions(workflow, node, additionalData, mode);
            returnFunctions.emit = (data) => {
                _1.WorkflowHelpers.saveStaticData(workflow);
                this.runWorkflow(workflowData, node, data, additionalData, mode).catch((err) => console.error(err));
            };
            return returnFunctions;
        });
    }
    async add(workflowId, workflowData) {
        if (this.activeWorkflows === null) {
            throw new Error(`The "activeWorkflows" instance did not get initialized yet.`);
        }
        let workflowInstance;
        try {
            if (workflowData === undefined) {
                workflowData = await _1.Db.collections.Workflow.findOne(workflowId);
            }
            if (!workflowData) {
                throw new Error(`Could not find workflow with id "${workflowId}".`);
            }
            const nodeTypes = _1.NodeTypes();
            workflowInstance = new n8n_workflow_1.Workflow({ id: workflowId, name: workflowData.name, nodes: workflowData.nodes, connections: workflowData.connections, active: workflowData.active, nodeTypes, staticData: workflowData.staticData, settings: workflowData.settings });
            const canBeActivated = workflowInstance.checkIfWorkflowCanBeActivated(['n8n-nodes-base.start']);
            if (canBeActivated === false) {
                throw new Error(`The workflow can not be activated because it does not contain any nodes which could start the workflow. Only workflows which have trigger or webhook nodes can be activated.`);
            }
            const mode = 'trigger';
            const credentials = await _1.WorkflowCredentials(workflowData.nodes);
            const additionalData = await _1.WorkflowExecuteAdditionalData.getBase(credentials);
            const getTriggerFunctions = this.getExecuteTriggerFunctions(workflowData, additionalData, mode);
            const getPollFunctions = this.getExecutePollFunctions(workflowData, additionalData, mode);
            await this.addWorkflowWebhooks(workflowInstance, additionalData, mode);
            if (workflowInstance.getTriggerNodes().length !== 0
                || workflowInstance.getPollNodes().length !== 0) {
                await this.activeWorkflows.add(workflowId, workflowInstance, additionalData, getTriggerFunctions, getPollFunctions);
            }
            if (this.activationErrors[workflowId] !== undefined) {
                delete this.activationErrors[workflowId];
            }
        }
        catch (error) {
            this.activationErrors[workflowId] = {
                time: new Date().getTime(),
                error: {
                    message: error.message,
                },
            };
            throw error;
        }
        await _1.WorkflowHelpers.saveStaticData(workflowInstance);
    }
    async remove(workflowId) {
        if (this.activeWorkflows !== null) {
            try {
                await this.removeWorkflowWebhooks(workflowId);
            }
            catch (error) {
                console.error(`Could not remove webhooks of workflow "${workflowId}" because of error: "${error.message}"`);
            }
            if (this.activationErrors[workflowId] !== undefined) {
                delete this.activationErrors[workflowId];
            }
            if (this.activeWorkflows.isActive(workflowId)) {
                this.activeWorkflows.remove(workflowId);
            }
            return;
        }
        throw new Error(`The "activeWorkflows" instance did not get initialized yet.`);
    }
}
exports.ActiveWorkflowRunner = ActiveWorkflowRunner;
let workflowRunnerInstance;
function getInstance() {
    if (workflowRunnerInstance === undefined) {
        workflowRunnerInstance = new ActiveWorkflowRunner();
    }
    return workflowRunnerInstance;
}
exports.getInstance = getInstance;
//# sourceMappingURL=ActiveWorkflowRunner.js.map