"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.collections = void 0;
const _1 = require("./");
const n8n_core_1 = require("n8n-core");
const typeorm_1 = require("typeorm");
const config = require("../config");
const databases_1 = require("./databases");
exports.collections = {
    Credentials: null,
    Execution: null,
    Workflow: null,
    Webhook: null,
};
const migrations_1 = require("./databases/postgresdb/migrations");
const migrations_2 = require("./databases/mysqldb/migrations");
const migrations_3 = require("./databases/sqlite/migrations");
const path = require("path");
async function init() {
    const dbType = await _1.GenericHelpers.getConfigValue('database.type');
    const n8nFolder = n8n_core_1.UserSettings.getUserN8nFolderPath();
    let entities;
    let connectionOptions;
    const entityPrefix = config.get('database.tablePrefix');
    switch (dbType) {
        case 'postgresdb':
            entities = databases_1.PostgresDb;
            const sslCa = await _1.GenericHelpers.getConfigValue('database.postgresdb.ssl.ca');
            const sslCert = await _1.GenericHelpers.getConfigValue('database.postgresdb.ssl.cert');
            const sslKey = await _1.GenericHelpers.getConfigValue('database.postgresdb.ssl.key');
            const sslRejectUnauthorized = await _1.GenericHelpers.getConfigValue('database.postgresdb.ssl.rejectUnauthorized');
            let ssl = undefined;
            if (sslCa !== '' || sslCert !== '' || sslKey !== '' || sslRejectUnauthorized !== true) {
                ssl = {
                    ca: sslCa || undefined,
                    cert: sslCert || undefined,
                    key: sslKey || undefined,
                    rejectUnauthorized: sslRejectUnauthorized,
                };
            }
            connectionOptions = {
                type: 'postgres',
                entityPrefix,
                database: await _1.GenericHelpers.getConfigValue('database.postgresdb.database'),
                host: await _1.GenericHelpers.getConfigValue('database.postgresdb.host'),
                password: await _1.GenericHelpers.getConfigValue('database.postgresdb.password'),
                port: await _1.GenericHelpers.getConfigValue('database.postgresdb.port'),
                username: await _1.GenericHelpers.getConfigValue('database.postgresdb.user'),
                schema: config.get('database.postgresdb.schema'),
                migrations: migrations_1.postgresMigrations,
                migrationsRun: true,
                migrationsTableName: `${entityPrefix}migrations`,
                ssl,
            };
            break;
        case 'mariadb':
        case 'mysqldb':
            entities = databases_1.MySQLDb;
            connectionOptions = {
                type: dbType === 'mysqldb' ? 'mysql' : 'mariadb',
                database: await _1.GenericHelpers.getConfigValue('database.mysqldb.database'),
                entityPrefix,
                host: await _1.GenericHelpers.getConfigValue('database.mysqldb.host'),
                password: await _1.GenericHelpers.getConfigValue('database.mysqldb.password'),
                port: await _1.GenericHelpers.getConfigValue('database.mysqldb.port'),
                username: await _1.GenericHelpers.getConfigValue('database.mysqldb.user'),
                migrations: migrations_2.mysqlMigrations,
                migrationsRun: true,
                migrationsTableName: `${entityPrefix}migrations`,
            };
            break;
        case 'sqlite':
            entities = databases_1.SQLite;
            connectionOptions = {
                type: 'sqlite',
                database: path.join(n8nFolder, 'database.sqlite'),
                entityPrefix,
                migrations: migrations_3.sqliteMigrations,
                migrationsRun: true,
                migrationsTableName: `${entityPrefix}migrations`,
            };
            break;
        default:
            throw new Error(`The database "${dbType}" is currently not supported!`);
    }
    Object.assign(connectionOptions, {
        entities: Object.values(entities),
        synchronize: false,
        logging: false,
    });
    const connection = await typeorm_1.createConnection(connectionOptions);
    await connection.runMigrations({
        transaction: 'none',
    });
    exports.collections.Credentials = typeorm_1.getRepository(entities.CredentialsEntity);
    exports.collections.Execution = typeorm_1.getRepository(entities.ExecutionEntity);
    exports.collections.Workflow = typeorm_1.getRepository(entities.WorkflowEntity);
    exports.collections.Webhook = typeorm_1.getRepository(entities.WebhookEntity);
    return exports.collections;
}
exports.init = init;
//# sourceMappingURL=Db.js.map