"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkflowRunnerProcess = void 0;
const _1 = require("./");
const n8n_core_1 = require("n8n-core");
const n8n_workflow_1 = require("n8n-workflow");
class WorkflowRunnerProcess {
    constructor() {
        this.startedAt = new Date();
    }
    async runWorkflow(inputData) {
        this.data = inputData;
        let className;
        let tempNode;
        let filePath;
        this.startedAt = new Date();
        const nodeTypesData = {};
        for (const nodeTypeName of Object.keys(this.data.nodeTypeData)) {
            className = this.data.nodeTypeData[nodeTypeName].className;
            filePath = this.data.nodeTypeData[nodeTypeName].sourcePath;
            const tempModule = require(filePath);
            try {
                tempNode = new tempModule[className]();
            }
            catch (error) {
                throw new Error(`Error loading node "${nodeTypeName}" from: "${filePath}"`);
            }
            nodeTypesData[nodeTypeName] = {
                type: tempNode,
                sourcePath: filePath,
            };
        }
        const nodeTypes = _1.NodeTypes();
        await nodeTypes.init(nodeTypesData);
        const credentialTypes = _1.CredentialTypes();
        await credentialTypes.init(inputData.credentialsTypeData);
        const credentialsOverwrites = _1.CredentialsOverwrites();
        await credentialsOverwrites.init(inputData.credentialsOverwrite);
        const externalHooks = _1.ExternalHooks();
        await externalHooks.init();
        this.workflow = new n8n_workflow_1.Workflow({ id: this.data.workflowData.id, name: this.data.workflowData.name, nodes: this.data.workflowData.nodes, connections: this.data.workflowData.connections, active: this.data.workflowData.active, nodeTypes, staticData: this.data.workflowData.staticData, settings: this.data.workflowData.settings });
        const additionalData = await _1.WorkflowExecuteAdditionalData.getBase(this.data.credentials);
        additionalData.hooks = this.getProcessForwardHooks();
        if (this.data.executionData !== undefined) {
            this.workflowExecute = new n8n_core_1.WorkflowExecute(additionalData, this.data.executionMode, this.data.executionData);
            return this.workflowExecute.processRunExecutionData(this.workflow);
        }
        else if (this.data.runData === undefined || this.data.startNodes === undefined || this.data.startNodes.length === 0 || this.data.destinationNode === undefined) {
            this.workflowExecute = new n8n_core_1.WorkflowExecute(additionalData, this.data.executionMode);
            return this.workflowExecute.run(this.workflow, undefined, this.data.destinationNode);
        }
        else {
            this.workflowExecute = new n8n_core_1.WorkflowExecute(additionalData, this.data.executionMode);
            return this.workflowExecute.runPartialWorkflow(this.workflow, this.data.runData, this.data.startNodes, this.data.destinationNode);
        }
    }
    sendHookToParentProcess(hook, parameters) {
        try {
            sendToParentProcess('processHook', {
                hook,
                parameters,
            });
        }
        catch (error) {
            console.error(`There was a problem sending hook: "${hook}"`);
            console.error('Parameters:');
            console.error(parameters);
            console.error('Error:');
            console.error(error);
        }
    }
    getProcessForwardHooks() {
        const hookFunctions = {
            nodeExecuteBefore: [
                async (nodeName) => {
                    this.sendHookToParentProcess('nodeExecuteBefore', [nodeName]);
                },
            ],
            nodeExecuteAfter: [
                async (nodeName, data) => {
                    this.sendHookToParentProcess('nodeExecuteAfter', [nodeName, data]);
                },
            ],
            workflowExecuteBefore: [
                async () => {
                    this.sendHookToParentProcess('workflowExecuteBefore', []);
                },
            ],
            workflowExecuteAfter: [
                async (fullRunData, newStaticData) => {
                    this.sendHookToParentProcess('workflowExecuteAfter', [fullRunData, newStaticData]);
                },
            ],
        };
        const preExecuteFunctions = _1.WorkflowExecuteAdditionalData.hookFunctionsPreExecute();
        for (const key of Object.keys(preExecuteFunctions)) {
            hookFunctions[key].push.apply(hookFunctions[key], preExecuteFunctions[key]);
        }
        return new n8n_workflow_1.WorkflowHooks(hookFunctions, this.data.executionMode, this.data.executionId, this.data.workflowData, { sessionId: this.data.sessionId, retryOf: this.data.retryOf });
    }
}
exports.WorkflowRunnerProcess = WorkflowRunnerProcess;
async function sendToParentProcess(type, data) {
    return new Promise((resolve, reject) => {
        process.send({
            type,
            data,
        }, (error) => {
            if (error) {
                return reject(error);
            }
            resolve();
        });
    });
}
const workflowRunner = new WorkflowRunnerProcess();
process.on('message', async (message) => {
    try {
        if (message.type === 'startWorkflow') {
            const runData = await workflowRunner.runWorkflow(message.data);
            await sendToParentProcess('end', {
                runData,
            });
            process.exit();
        }
        else if (message.type === 'stopExecution' || message.type === 'timeout') {
            let runData;
            if (workflowRunner.workflowExecute !== undefined) {
                runData = workflowRunner.workflowExecute.getFullRunData(workflowRunner.startedAt);
                const timeOutError = message.type === 'timeout' ? { message: 'Workflow execution timed out!' } : undefined;
                await workflowRunner.workflowExecute.processSuccessExecution(workflowRunner.startedAt, workflowRunner.workflow, timeOutError);
            }
            else {
                runData = {
                    data: {
                        resultData: {
                            runData: {},
                        },
                    },
                    finished: message.type !== 'timeout',
                    mode: workflowRunner.data.executionMode,
                    startedAt: workflowRunner.startedAt,
                    stoppedAt: new Date(),
                };
                workflowRunner.sendHookToParentProcess('workflowExecuteAfter', [runData]);
            }
            await sendToParentProcess(message.type === 'timeout' ? message.type : 'end', {
                runData,
            });
            process.exit();
        }
    }
    catch (error) {
        const executionError = {
            message: error.message,
            stack: error.stack,
        };
        await sendToParentProcess('processError', {
            executionError,
        });
        process.exit();
    }
});
//# sourceMappingURL=WorkflowRunnerProcess.js.map