"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Box = void 0;
const n8n_core_1 = require("n8n-core");
const GenericFunctions_1 = require("./GenericFunctions");
const FileDescription_1 = require("./FileDescription");
const FolderDescription_1 = require("./FolderDescription");
const moment = require("moment-timezone");
class Box {
    constructor() {
        this.description = {
            displayName: 'Box',
            name: 'box',
            icon: 'file:box.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Box API',
            defaults: {
                name: 'Box',
                color: '#00aeef',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'boxOAuth2Api',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'File',
                            value: 'file',
                        },
                        {
                            name: 'Folder',
                            value: 'folder',
                        },
                    ],
                    default: 'file',
                    description: 'The resource to operate on.',
                },
                ...FileDescription_1.fileOperations,
                ...FileDescription_1.fileFields,
                ...FolderDescription_1.folderOperations,
                ...FolderDescription_1.folderFields,
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < length; i++) {
            if (resource === 'file') {
                if (operation === 'copy') {
                    const fileId = this.getNodeParameter('fileId', i);
                    const parentId = this.getNodeParameter('parentId', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const body = {};
                    if (additionalFields.name) {
                        body.name = additionalFields.name;
                    }
                    if (parentId) {
                        body.parent = { id: parentId };
                    }
                    else {
                        body.parent = { id: 0 };
                    }
                    if (additionalFields.fields) {
                        qs.fields = additionalFields.fields;
                    }
                    if (additionalFields.version) {
                        body.version = additionalFields.version;
                    }
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'POST', `/files/${fileId}/copy`, body, qs);
                    returnData.push(responseData);
                }
                if (operation === 'delete') {
                    const fileId = this.getNodeParameter('fileId', i);
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'DELETE', `/files/${fileId}`);
                    responseData = { success: true };
                    returnData.push(responseData);
                }
                if (operation === 'download') {
                    const fileId = this.getNodeParameter('fileId', i);
                    const dataPropertyNameDownload = this.getNodeParameter('binaryPropertyName', i);
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'GET', `/files/${fileId}`);
                    const fileName = responseData.name;
                    let mimeType;
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'GET', `/files/${fileId}/content`, {}, {}, undefined, { resolveWithFullResponse: true });
                    const newItem = {
                        json: items[i].json,
                        binary: {},
                    };
                    if (mimeType === undefined && responseData.headers['content-type']) {
                        mimeType = responseData.headers['content-type'];
                    }
                    if (items[i].binary !== undefined) {
                        Object.assign(newItem.binary, items[i].binary);
                    }
                    items[i] = newItem;
                    const data = Buffer.from(responseData.body);
                    items[i].binary[dataPropertyNameDownload] = await this.helpers.prepareBinaryData(data, fileName, mimeType);
                }
                if (operation === 'get') {
                    const fileId = this.getNodeParameter('fileId', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    if (additionalFields.fields) {
                        qs.fields = additionalFields.fields;
                    }
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'GET', `/files/${fileId}`, {}, qs);
                    returnData.push(responseData);
                }
                if (operation === 'search') {
                    const query = this.getNodeParameter('query', i);
                    const returnAll = this.getNodeParameter('returnAll', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const timezone = this.getTimezone();
                    qs.type = 'file';
                    qs.query = query;
                    Object.assign(qs, additionalFields);
                    if (qs.content_types) {
                        qs.content_types = qs.content_types.split(',');
                    }
                    if (additionalFields.createdRangeUi) {
                        const createdRangeValues = additionalFields.createdRangeUi.createdRangeValuesUi;
                        if (createdRangeValues) {
                            qs.created_at_range = `${moment.tz(createdRangeValues.from, timezone).format()},${moment.tz(createdRangeValues.to, timezone).format()}`;
                        }
                        delete qs.createdRangeUi;
                    }
                    if (additionalFields.updatedRangeUi) {
                        const updateRangeValues = additionalFields.updatedRangeUi.updatedRangeValuesUi;
                        if (updateRangeValues) {
                            qs.updated_at_range = `${moment.tz(updateRangeValues.from, timezone).format()},${moment.tz(updateRangeValues.to, timezone).format()}`;
                        }
                        delete qs.updatedRangeUi;
                    }
                    if (returnAll) {
                        responseData = await GenericFunctions_1.boxApiRequestAllItems.call(this, 'entries', 'GET', `/search`, {}, qs);
                    }
                    else {
                        qs.limit = this.getNodeParameter('limit', i);
                        responseData = await GenericFunctions_1.boxApiRequest.call(this, 'GET', `/search`, {}, qs);
                        responseData = responseData.entries;
                    }
                    returnData.push.apply(returnData, responseData);
                }
                if (operation === 'upload') {
                    const parentId = this.getNodeParameter('parentId', i);
                    const isBinaryData = this.getNodeParameter('binaryData', i);
                    const fileName = this.getNodeParameter('fileName', i);
                    const attributes = {};
                    if (parentId !== '') {
                        attributes['parent'] = { id: parentId };
                    }
                    else {
                        attributes['parent'] = { id: 0 };
                    }
                    if (isBinaryData) {
                        const binaryPropertyName = this.getNodeParameter('binaryPropertyName', 0);
                        if (items[i].binary === undefined) {
                            throw new Error('No binary data exists on item!');
                        }
                        if (items[i].binary[binaryPropertyName] === undefined) {
                            throw new Error(`No binary data property "${binaryPropertyName}" does not exists on item!`);
                        }
                        const binaryData = items[i].binary[binaryPropertyName];
                        const body = {};
                        attributes['name'] = fileName || binaryData.fileName;
                        body['attributes'] = JSON.stringify(attributes);
                        body['file'] = {
                            value: Buffer.from(binaryData.data, n8n_core_1.BINARY_ENCODING),
                            options: {
                                filename: binaryData.fileName,
                                contentType: binaryData.mimeType,
                            },
                        };
                        responseData = await GenericFunctions_1.boxApiRequest.call(this, 'POST', '', {}, {}, 'https://upload.box.com/api/2.0/files/content', { formData: body });
                        returnData.push.apply(returnData, responseData.entries);
                    }
                    else {
                        const content = this.getNodeParameter('fileContent', i);
                        if (fileName === '') {
                            throw new Error('File name must be set!');
                        }
                        attributes['name'] = fileName;
                        const body = {};
                        body['attributes'] = JSON.stringify(attributes);
                        body['file'] = {
                            value: Buffer.from(content),
                            options: {
                                filename: fileName,
                                contentType: 'text/plain',
                            },
                        };
                        responseData = await GenericFunctions_1.boxApiRequest.call(this, 'POST', '', {}, {}, 'https://upload.box.com/api/2.0/files/content', { formData: body });
                        returnData.push.apply(returnData, responseData.entries);
                    }
                }
            }
            if (resource === 'folder') {
                if (operation === 'create') {
                    const name = this.getNodeParameter('name', i);
                    const parentId = this.getNodeParameter('parentId', i);
                    const options = this.getNodeParameter('options', i);
                    const body = {
                        name,
                    };
                    if (parentId) {
                        body.parent = { id: parentId };
                    }
                    else {
                        body.parent = { id: 0 };
                    }
                    if (options.access) {
                        body.folder_upload_email = {
                            access: options.access,
                        };
                    }
                    if (options.fields) {
                        qs.fields = options.fields;
                    }
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'POST', '/folders', body, qs);
                    returnData.push(responseData);
                }
                if (operation === 'delete') {
                    const folderId = this.getNodeParameter('folderId', i);
                    const recursive = this.getNodeParameter('recursive', i);
                    qs.recursive = recursive;
                    responseData = await GenericFunctions_1.boxApiRequest.call(this, 'DELETE', `/folders/${folderId}`, qs);
                    responseData = { success: true };
                    returnData.push(responseData);
                }
                if (operation === 'search') {
                    const query = this.getNodeParameter('query', i);
                    const returnAll = this.getNodeParameter('returnAll', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const timezone = this.getTimezone();
                    qs.type = 'folder';
                    qs.query = query;
                    Object.assign(qs, additionalFields);
                    if (qs.content_types) {
                        qs.content_types = qs.content_types.split(',');
                    }
                    if (additionalFields.createdRangeUi) {
                        const createdRangeValues = additionalFields.createdRangeUi.createdRangeValuesUi;
                        if (createdRangeValues) {
                            qs.created_at_range = `${moment.tz(createdRangeValues.from, timezone).format()},${moment.tz(createdRangeValues.to, timezone).format()}`;
                        }
                        delete qs.createdRangeUi;
                    }
                    if (additionalFields.updatedRangeUi) {
                        const updateRangeValues = additionalFields.updatedRangeUi.updatedRangeValuesUi;
                        if (updateRangeValues) {
                            qs.updated_at_range = `${moment.tz(updateRangeValues.from, timezone).format()},${moment.tz(updateRangeValues.to, timezone).format()}`;
                        }
                        delete qs.updatedRangeUi;
                    }
                    if (returnAll) {
                        responseData = await GenericFunctions_1.boxApiRequestAllItems.call(this, 'entries', 'GET', `/search`, {}, qs);
                    }
                    else {
                        qs.limit = this.getNodeParameter('limit', i);
                        responseData = await GenericFunctions_1.boxApiRequest.call(this, 'GET', `/search`, {}, qs);
                        responseData = responseData.entries;
                    }
                    returnData.push.apply(returnData, responseData);
                }
            }
        }
        if (resource === 'file' && operation === 'download') {
            return this.prepareOutputData(items);
        }
        else {
            return [this.helpers.returnJsonArray(returnData)];
        }
    }
}
exports.Box = Box;
//# sourceMappingURL=Box.node.js.map