"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmailSend = void 0;
const n8n_core_1 = require("n8n-core");
const nodemailer_1 = require("nodemailer");
class EmailSend {
    constructor() {
        this.description = {
            displayName: 'Send Email',
            name: 'emailSend',
            icon: 'fa:envelope',
            group: ['output'],
            version: 1,
            description: 'Sends an Email',
            defaults: {
                name: 'Send Email',
                color: '#00bb88',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'smtp',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'From Email',
                    name: 'fromEmail',
                    type: 'string',
                    default: '',
                    required: true,
                    placeholder: 'admin@example.com',
                    description: 'Email address of the sender optional with name.',
                },
                {
                    displayName: 'To Email',
                    name: 'toEmail',
                    type: 'string',
                    default: '',
                    required: true,
                    placeholder: 'info@example.com',
                    description: 'Email address of the recipient.',
                },
                {
                    displayName: 'CC Email',
                    name: 'ccEmail',
                    type: 'string',
                    default: '',
                    placeholder: 'cc@example.com',
                    description: 'Email address of CC recipient.',
                },
                {
                    displayName: 'BCC Email',
                    name: 'bccEmail',
                    type: 'string',
                    default: '',
                    placeholder: 'bcc@example.com',
                    description: 'Email address of BCC recipient.',
                },
                {
                    displayName: 'Subject',
                    name: 'subject',
                    type: 'string',
                    default: '',
                    placeholder: 'My subject line',
                    description: 'Subject line of the email.',
                },
                {
                    displayName: 'Text',
                    name: 'text',
                    type: 'string',
                    typeOptions: {
                        alwaysOpenEditWindow: true,
                        rows: 5,
                    },
                    default: '',
                    description: 'Plain text message of email.',
                },
                {
                    displayName: 'HTML',
                    name: 'html',
                    type: 'string',
                    typeOptions: {
                        rows: 5,
                    },
                    default: '',
                    description: 'HTML text message of email.',
                },
                {
                    displayName: 'Attachments',
                    name: 'attachments',
                    type: 'string',
                    default: '',
                    description: 'Name of the binary properties which contain<br />data which should be added to email as attachment.<br />Multiple ones can be comma separated.',
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Ignore SSL Issues',
                            name: 'allowUnauthorizedCerts',
                            type: 'boolean',
                            default: false,
                            description: 'Do connect even if SSL certificate validation is not possible.',
                        },
                    ],
                },
            ],
        };
    }
    async executeSingle() {
        const item = this.getInputData();
        const fromEmail = this.getNodeParameter('fromEmail');
        const toEmail = this.getNodeParameter('toEmail');
        const ccEmail = this.getNodeParameter('ccEmail');
        const bccEmail = this.getNodeParameter('bccEmail');
        const subject = this.getNodeParameter('subject');
        const text = this.getNodeParameter('text');
        const html = this.getNodeParameter('html');
        const attachmentPropertyString = this.getNodeParameter('attachments');
        const options = this.getNodeParameter('options', {});
        const credentials = this.getCredentials('smtp');
        if (credentials === undefined) {
            throw new Error('No credentials got returned!');
        }
        const connectionOptions = {
            host: credentials.host,
            port: credentials.port,
            secure: credentials.secure,
        };
        if (credentials.user || credentials.password) {
            connectionOptions.auth = {
                user: credentials.user,
                pass: credentials.password,
            };
        }
        if (options.allowUnauthorizedCerts === true) {
            connectionOptions.tls = {
                rejectUnauthorized: false,
            };
        }
        const transporter = nodemailer_1.createTransport(connectionOptions);
        const mailOptions = {
            from: fromEmail,
            to: toEmail,
            cc: ccEmail,
            bcc: bccEmail,
            subject,
            text,
            html,
        };
        if (attachmentPropertyString && item.binary) {
            const attachments = [];
            const attachmentProperties = attachmentPropertyString.split(',').map((propertyName) => {
                return propertyName.trim();
            });
            for (const propertyName of attachmentProperties) {
                if (!item.binary.hasOwnProperty(propertyName)) {
                    continue;
                }
                attachments.push({
                    filename: item.binary[propertyName].fileName || 'unknown',
                    content: Buffer.from(item.binary[propertyName].data, n8n_core_1.BINARY_ENCODING),
                });
            }
            if (attachments.length) {
                mailOptions.attachments = attachments;
            }
        }
        const info = await transporter.sendMail(mailOptions);
        return { json: info };
    }
}
exports.EmailSend = EmailSend;
//# sourceMappingURL=EmailSend.node.js.map