"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecuteWorkflow = void 0;
const fs_1 = require("fs");
const util_1 = require("util");
const fsReadFileAsync = util_1.promisify(fs_1.readFile);
class ExecuteWorkflow {
    constructor() {
        this.description = {
            displayName: 'Execute Workflow',
            name: 'executeWorkflow',
            icon: 'fa:network-wired',
            group: ['transform'],
            version: 1,
            subtitle: '={{"Workflow: " + $parameter["workflowId"]}}',
            description: 'Execute another workflow',
            defaults: {
                name: 'Execute Workflow',
                color: '#ff6d5a',
            },
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Source',
                    name: 'source',
                    type: 'options',
                    options: [
                        {
                            name: 'Database',
                            value: 'database',
                            description: 'Load the workflow from the database by ID.',
                        },
                        {
                            name: 'Local File',
                            value: 'localFile',
                            description: 'Load the workflow from a locally saved file.',
                        },
                        {
                            name: 'Parameter',
                            value: 'parameter',
                            description: 'Load the workflow from a parameter.',
                        },
                        {
                            name: 'URL',
                            value: 'url',
                            description: 'Load the workflow from an URL.',
                        },
                    ],
                    default: 'database',
                    description: 'Where to get the workflow to execute from.',
                },
                {
                    displayName: 'Workflow ID',
                    name: 'workflowId',
                    type: 'string',
                    displayOptions: {
                        show: {
                            source: [
                                'database',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'The workflow to execute.',
                },
                {
                    displayName: 'Workflow Path',
                    name: 'workflowPath',
                    type: 'string',
                    displayOptions: {
                        show: {
                            source: [
                                'localFile',
                            ],
                        },
                    },
                    default: '',
                    placeholder: '/data/workflow.json',
                    required: true,
                    description: 'The path to local JSON workflow file to execute.',
                },
                {
                    displayName: 'Workflow JSON',
                    name: 'workflowJson',
                    type: 'string',
                    typeOptions: {
                        alwaysOpenEditWindow: true,
                        editor: 'code',
                        rows: 10,
                    },
                    displayOptions: {
                        show: {
                            source: [
                                'parameter',
                            ],
                        },
                    },
                    default: '\n\n\n',
                    required: true,
                    description: 'The workflow JSON code to execute.',
                },
                {
                    displayName: 'Workflow URL',
                    name: 'workflowUrl',
                    type: 'string',
                    displayOptions: {
                        show: {
                            source: [
                                'url',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'https://example.com/workflow.json',
                    required: true,
                    description: 'The URL from which to load the workflow from.',
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const source = this.getNodeParameter('source', 0);
        const workflowInfo = {};
        if (source === 'database') {
            workflowInfo.id = this.getNodeParameter('workflowId', 0);
        }
        else if (source === 'localFile') {
            const workflowPath = this.getNodeParameter('workflowPath', 0);
            let workflowJson;
            try {
                workflowJson = await fsReadFileAsync(workflowPath, { encoding: 'utf8' });
            }
            catch (error) {
                if (error.code === 'ENOENT') {
                    throw new Error(`The file "${workflowPath}" could not be found.`);
                }
                throw error;
            }
            workflowInfo.code = JSON.parse(workflowJson);
        }
        else if (source === 'parameter') {
            const workflowJson = this.getNodeParameter('workflowJson', 0);
            workflowInfo.code = JSON.parse(workflowJson);
        }
        else if (source === 'url') {
            const workflowUrl = this.getNodeParameter('workflowUrl', 0);
            const requestOptions = {
                headers: {
                    'accept': 'application/json,text/*;q=0.99',
                },
                method: 'GET',
                uri: workflowUrl,
                json: true,
                gzip: true,
            };
            const response = await this.helpers.request(requestOptions);
            workflowInfo.code = response;
        }
        const receivedData = await this.executeWorkflow(workflowInfo, items);
        return receivedData;
    }
}
exports.ExecuteWorkflow = ExecuteWorkflow;
//# sourceMappingURL=ExecuteWorkflow.node.js.map