"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ghost = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const PostDescription_1 = require("./PostDescription");
const moment = require("moment-timezone");
class Ghost {
    constructor() {
        this.description = {
            displayName: 'Ghost',
            name: 'ghost',
            icon: 'file:ghost.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Ghost API.',
            defaults: {
                name: 'Ghost',
                color: '#15212a',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'ghostAdminApi',
                    required: true,
                    displayOptions: {
                        show: {
                            source: [
                                'adminApi',
                            ],
                        },
                    },
                },
                {
                    name: 'ghostContentApi',
                    required: true,
                    displayOptions: {
                        show: {
                            source: [
                                'contentApi',
                            ],
                        },
                    },
                },
            ],
            properties: [
                {
                    displayName: 'Source',
                    name: 'source',
                    type: 'options',
                    description: 'Pick where your data comes from, Content or Admin API',
                    options: [
                        {
                            name: 'Admin API',
                            value: 'adminApi',
                        },
                        {
                            name: 'Content API',
                            value: 'contentApi',
                        },
                    ],
                    default: 'contentApi',
                },
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Post',
                            value: 'post',
                        },
                    ],
                    default: 'post',
                    description: 'The resource to operate on.',
                },
                ...PostDescription_1.postOperations,
                ...PostDescription_1.postFields,
            ],
        };
        this.methods = {
            loadOptions: {
                async getAuthors() {
                    const returnData = [];
                    const users = await GenericFunctions_1.ghostApiRequestAllItems.call(this, 'users', 'GET', `/admin/users`);
                    for (const user of users) {
                        returnData.push({
                            name: user.name,
                            value: user.id,
                        });
                    }
                    return returnData;
                },
                async getTags() {
                    const returnData = [];
                    const tags = await GenericFunctions_1.ghostApiRequestAllItems.call(this, 'tags', 'GET', `/admin/tags`);
                    for (const tag of tags) {
                        returnData.push({
                            name: tag.name,
                            value: tag.id,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const timezone = this.getTimezone();
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        const source = this.getNodeParameter('source', 0);
        if (source === 'contentApi') {
            if (resource === 'post') {
                if (operation === 'get') {
                    for (let i = 0; i < items.length; i++) {
                        const by = this.getNodeParameter('by', i);
                        const identifier = this.getNodeParameter('identifier', i);
                        const options = this.getNodeParameter('options', i);
                        Object.assign(qs, options);
                        let endpoint;
                        if (by === 'slug') {
                            endpoint = `/content/posts/slug/${identifier}`;
                        }
                        else {
                            endpoint = `/content/posts/${identifier}`;
                        }
                        responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'GET', endpoint, {}, qs);
                        returnData.push.apply(returnData, responseData.posts);
                    }
                }
                if (operation === 'getAll') {
                    for (let i = 0; i < items.length; i++) {
                        const returnAll = this.getNodeParameter('returnAll', 0);
                        const options = this.getNodeParameter('options', i);
                        Object.assign(qs, options);
                        if (returnAll) {
                            responseData = await GenericFunctions_1.ghostApiRequestAllItems.call(this, 'posts', 'GET', '/content/posts', {}, qs);
                        }
                        else {
                            qs.limit = this.getNodeParameter('limit', 0);
                            responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'GET', '/content/posts', {}, qs);
                            responseData = responseData.posts;
                        }
                        returnData.push.apply(returnData, responseData);
                    }
                }
            }
        }
        if (source === 'adminApi') {
            if (resource === 'post') {
                if (operation === 'create') {
                    for (let i = 0; i < length; i++) {
                        const title = this.getNodeParameter('title', i);
                        const contentFormat = this.getNodeParameter('contentFormat', i);
                        const content = this.getNodeParameter('content', i);
                        const additionalFields = this.getNodeParameter('additionalFields', i);
                        const post = {
                            title,
                        };
                        if (contentFormat === 'html') {
                            post.html = content;
                            qs.source = 'html';
                        }
                        else {
                            const mobileDoc = GenericFunctions_1.validateJSON(content);
                            if (mobileDoc === undefined) {
                                throw new Error('Content must be a valid JSON');
                            }
                            post.mobiledoc = content;
                        }
                        delete post.content;
                        Object.assign(post, additionalFields);
                        if (post.published_at) {
                            post.published_at = moment.tz(post.published_at, timezone).utc().format();
                        }
                        if (post.status === 'scheduled' && post.published_at === undefined) {
                            throw new Error('Published at must be define when status is scheduled');
                        }
                        responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'POST', '/admin/posts', { posts: [post] }, qs);
                        returnData.push.apply(returnData, responseData.posts);
                    }
                }
                if (operation === 'delete') {
                    for (let i = 0; i < length; i++) {
                        const postId = this.getNodeParameter('postId', i);
                        responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'DELETE', `/admin/posts/${postId}`);
                        returnData.push({ success: true });
                    }
                }
                if (operation === 'get') {
                    for (let i = 0; i < length; i++) {
                        const by = this.getNodeParameter('by', i);
                        const identifier = this.getNodeParameter('identifier', i);
                        const options = this.getNodeParameter('options', i);
                        Object.assign(qs, options);
                        let endpoint;
                        if (by === 'slug') {
                            endpoint = `/admin/posts/slug/${identifier}`;
                        }
                        else {
                            endpoint = `/admin/posts/${identifier}`;
                        }
                        responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'GET', endpoint, {}, qs);
                        returnData.push.apply(returnData, responseData.posts);
                    }
                }
                if (operation === 'getAll') {
                    for (let i = 0; i < length; i++) {
                        const returnAll = this.getNodeParameter('returnAll', 0);
                        const options = this.getNodeParameter('options', i);
                        Object.assign(qs, options);
                        if (returnAll) {
                            responseData = await GenericFunctions_1.ghostApiRequestAllItems.call(this, 'posts', 'GET', '/admin/posts', {}, qs);
                        }
                        else {
                            qs.limit = this.getNodeParameter('limit', 0);
                            responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'GET', '/admin/posts', {}, qs);
                            responseData = responseData.posts;
                        }
                        returnData.push.apply(returnData, responseData);
                    }
                }
                if (operation === 'update') {
                    for (let i = 0; i < length; i++) {
                        const postId = this.getNodeParameter('postId', i);
                        const contentFormat = this.getNodeParameter('contentFormat', i);
                        const updateFields = this.getNodeParameter('updateFields', i);
                        const post = {};
                        if (contentFormat === 'html') {
                            post.html = updateFields.content || '';
                            qs.source = 'html';
                            delete updateFields.content;
                        }
                        else {
                            const mobileDoc = GenericFunctions_1.validateJSON(updateFields.contentJson || undefined);
                            if (mobileDoc === undefined) {
                                throw new Error('Content must be a valid JSON');
                            }
                            post.mobiledoc = updateFields.contentJson;
                            delete updateFields.contentJson;
                        }
                        Object.assign(post, updateFields);
                        const { posts } = await GenericFunctions_1.ghostApiRequest.call(this, 'GET', `/admin/posts/${postId}`, {}, { fields: 'id, updated_at' });
                        if (post.published_at) {
                            post.published_at = moment.tz(post.published_at, timezone).utc().format();
                        }
                        if (post.status === 'scheduled' && post.published_at === undefined) {
                            throw new Error('Published at must be define when status is scheduled');
                        }
                        post.updated_at = posts[0].updated_at;
                        responseData = await GenericFunctions_1.ghostApiRequest.call(this, 'PUT', `/admin/posts/${postId}`, { posts: [post] }, qs);
                        returnData.push.apply(returnData, responseData.posts);
                    }
                }
            }
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.Ghost = Ghost;
//# sourceMappingURL=Ghost.node.js.map