"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.githubApiRequestAllItems = exports.getFileSha = exports.githubApiRequest = void 0;
async function githubApiRequest(method, endpoint, body, query, option = {}) {
    const options = {
        method,
        headers: {
            'User-Agent': 'n8n',
        },
        body,
        qs: query,
        uri: '',
        json: true,
    };
    if (Object.keys(option).length !== 0) {
        Object.assign(options, option);
    }
    try {
        const authenticationMethod = this.getNodeParameter('authentication', 0, 'accessToken');
        if (authenticationMethod === 'accessToken') {
            const credentials = this.getCredentials('githubApi');
            if (credentials === undefined) {
                throw new Error('No credentials got returned!');
            }
            const baseUrl = credentials.server || 'https://api.github.com';
            options.uri = `${baseUrl}${endpoint}`;
            options.headers.Authorization = `token ${credentials.accessToken}`;
            return await this.helpers.request(options);
        }
        else {
            const credentials = this.getCredentials('githubOAuth2Api');
            const baseUrl = credentials.server || 'https://api.github.com';
            options.uri = `${baseUrl}${endpoint}`;
            return await this.helpers.requestOAuth2.call(this, 'githubOAuth2Api', options);
        }
    }
    catch (error) {
        if (error.statusCode === 401) {
            throw new Error('The Github credentials are not valid!');
        }
        if (error.response && error.response.body && error.response.body.message) {
            throw new Error(`Github error response [${error.statusCode}]: ${error.response.body.message}`);
        }
        throw error;
    }
}
exports.githubApiRequest = githubApiRequest;
async function getFileSha(owner, repository, filePath, branch) {
    const getBody = {};
    if (branch !== undefined) {
        getBody.branch = branch;
    }
    const getEndpoint = `/repos/${owner}/${repository}/contents/${encodeURI(filePath)}`;
    const responseData = await githubApiRequest.call(this, 'GET', getEndpoint, getBody, {});
    if (responseData.sha === undefined) {
        throw new Error('Could not get the SHA of the file.');
    }
    return responseData.sha;
}
exports.getFileSha = getFileSha;
async function githubApiRequestAllItems(method, endpoint, body = {}, query = {}) {
    const returnData = [];
    let responseData;
    query.per_page = 100;
    query.page = 1;
    do {
        responseData = await githubApiRequest.call(this, method, endpoint, body, query, { resolveWithFullResponse: true });
        query.page++;
        returnData.push.apply(returnData, responseData.body);
    } while (responseData.headers.link && responseData.headers.link.includes('next'));
    return returnData;
}
exports.githubApiRequestAllItems = githubApiRequestAllItems;
//# sourceMappingURL=GenericFunctions.js.map