"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HubspotTrigger = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const crypto_1 = require("crypto");
const change_case_1 = require("change-case");
class HubspotTrigger {
    constructor() {
        this.description = {
            displayName: 'HubSpot Trigger',
            name: 'hubspotTrigger',
            icon: 'file:hubspot.svg',
            group: ['trigger'],
            version: 1,
            description: 'Starts the workflow when HubSpot events occur.',
            defaults: {
                name: 'Hubspot Trigger',
                color: '#ff7f64',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [
                {
                    name: 'hubspotDeveloperApi',
                    required: true,
                },
            ],
            webhooks: [
                {
                    name: 'default',
                    httpMethod: 'POST',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
                {
                    name: 'setup',
                    httpMethod: 'GET',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
            ],
            properties: [
                {
                    displayName: 'Events',
                    name: 'eventsUi',
                    type: 'fixedCollection',
                    typeOptions: {
                        multipleValues: true,
                    },
                    placeholder: 'Add Event',
                    default: {},
                    options: [
                        {
                            displayName: 'Event',
                            name: 'eventValues',
                            values: [
                                {
                                    displayName: 'Name',
                                    name: 'name',
                                    type: 'options',
                                    options: [
                                        {
                                            name: 'Contact Created',
                                            value: 'contact.creation',
                                            description: `To get notified if any contact is created in a customer's account.`,
                                        },
                                        {
                                            name: 'Contact Deleted',
                                            value: 'contact.deletion',
                                            description: `To get notified if any contact is deleted in a customer's account.`,
                                        },
                                        {
                                            name: 'Contact Privacy Deleted',
                                            value: 'contact.privacyDeletion',
                                            description: `To get notified if a contact is deleted for privacy compliance reasons. `,
                                        },
                                        {
                                            name: 'Contact Property Changed',
                                            value: 'contact.propertyChange',
                                            description: `to get notified if a specified property is changed for any contact in a customer's account. `,
                                        },
                                        {
                                            name: 'Company Created',
                                            value: 'company.creation',
                                            description: `To get notified if any company is created in a customer's account.`,
                                        },
                                        {
                                            name: 'Company Deleted',
                                            value: 'company.deletion',
                                            description: `To get notified if any company is deleted in a customer's account.`,
                                        },
                                        {
                                            name: 'Company Property Changed',
                                            value: 'company.propertyChange',
                                            description: `To get notified if a specified property is changed for any company in a customer's account.`,
                                        },
                                        {
                                            name: 'Deal Created',
                                            value: 'deal.creation',
                                            description: `To get notified if any deal is created in a customer's account.`,
                                        },
                                        {
                                            name: 'Deal Deleted',
                                            value: 'deal.deletion',
                                            description: `To get notified if any deal is deleted in a customer's account.`,
                                        },
                                        {
                                            name: 'Deal Property Changed',
                                            value: 'deal.propertyChange',
                                            description: `To get notified if a specified property is changed for any deal in a customer's account.`,
                                        },
                                    ],
                                    default: 'contact.creation',
                                    required: true,
                                },
                                {
                                    displayName: 'Property',
                                    name: 'property',
                                    type: 'options',
                                    typeOptions: {
                                        loadOptionsMethod: 'getContactProperties',
                                    },
                                    displayOptions: {
                                        show: {
                                            name: [
                                                'contact.propertyChange',
                                            ],
                                        },
                                    },
                                    default: '',
                                    required: true,
                                },
                                {
                                    displayName: 'Property',
                                    name: 'property',
                                    type: 'options',
                                    typeOptions: {
                                        loadOptionsMethod: 'getCompanyProperties',
                                    },
                                    displayOptions: {
                                        show: {
                                            name: [
                                                'company.propertyChange',
                                            ],
                                        },
                                    },
                                    default: '',
                                    required: true,
                                },
                                {
                                    displayName: 'Property',
                                    name: 'property',
                                    type: 'options',
                                    typeOptions: {
                                        loadOptionsMethod: 'getDealProperties',
                                    },
                                    displayOptions: {
                                        show: {
                                            name: [
                                                'deal.propertyChange',
                                            ],
                                        },
                                    },
                                    default: '',
                                    required: true,
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    default: {},
                    options: [
                        {
                            displayName: 'Max Concurrent Requests',
                            name: 'maxConcurrentRequests',
                            type: 'number',
                            typeOptions: {
                                minValue: 5,
                            },
                            default: 5,
                        },
                    ],
                },
            ],
        };
        this.methods = {
            loadOptions: {
                async getContactProperties() {
                    const returnData = [];
                    for (const field of GenericFunctions_1.contactFields) {
                        returnData.push({
                            name: change_case_1.capitalCase(field.label),
                            value: field.id,
                        });
                    }
                    returnData.sort((a, b) => {
                        if (a.name < b.name) {
                            return -1;
                        }
                        if (a.name > b.name) {
                            return 1;
                        }
                        return 0;
                    });
                    return returnData;
                },
                async getCompanyProperties() {
                    const returnData = [];
                    for (const field of GenericFunctions_1.companyFields) {
                        returnData.push({
                            name: change_case_1.capitalCase(field.label),
                            value: field.id,
                        });
                    }
                    returnData.sort((a, b) => {
                        if (a.name < b.name) {
                            return -1;
                        }
                        if (a.name > b.name) {
                            return 1;
                        }
                        return 0;
                    });
                    return returnData;
                },
                async getDealProperties() {
                    const returnData = [];
                    for (const field of GenericFunctions_1.dealFields) {
                        returnData.push({
                            name: change_case_1.capitalCase(field.label),
                            value: field.id,
                        });
                    }
                    returnData.sort((a, b) => {
                        if (a.name < b.name) {
                            return -1;
                        }
                        if (a.name > b.name) {
                            return 1;
                        }
                        return 0;
                    });
                    return returnData;
                },
            },
        };
        this.webhookMethods = {
            default: {
                async checkExists() {
                    const currentWebhookUrl = this.getNodeWebhookUrl('default');
                    const { appId } = this.getCredentials('hubspotDeveloperApi');
                    try {
                        const { targetUrl } = await GenericFunctions_1.hubspotApiRequest.call(this, 'GET', `/webhooks/v3/${appId}/settings`, {});
                        if (targetUrl !== currentWebhookUrl) {
                            throw new Error(`The APP ID ${appId} already has a target url ${targetUrl}. Delete it or use another APP ID before executing the trigger. Due to Hubspot API limitations, you can have just one trigger per APP.`);
                        }
                    }
                    catch (error) {
                        if (error.statusCode === 404) {
                            return false;
                        }
                    }
                    const { results: subscriptions } = await GenericFunctions_1.hubspotApiRequest.call(this, 'GET', `/webhooks/v3/${appId}/subscriptions`, {});
                    for (const subscription of subscriptions) {
                        await GenericFunctions_1.hubspotApiRequest.call(this, 'DELETE', `/webhooks/v3/${appId}/subscriptions/${subscription.id}`, {});
                    }
                    await GenericFunctions_1.hubspotApiRequest.call(this, 'DELETE', `/webhooks/v3/${appId}/settings`, {});
                    return false;
                },
                async create() {
                    const webhookUrl = this.getNodeWebhookUrl('default');
                    const { appId } = this.getCredentials('hubspotDeveloperApi');
                    const events = (this.getNodeParameter('eventsUi') || {}).eventValues || [];
                    const additionalFields = this.getNodeParameter('additionalFields');
                    let endpoint = `/webhooks/v3/${appId}/settings`;
                    let body = {
                        targetUrl: webhookUrl,
                        maxConcurrentRequests: additionalFields.maxConcurrentRequests || 5,
                    };
                    await GenericFunctions_1.hubspotApiRequest.call(this, 'PUT', endpoint, body);
                    endpoint = `/webhooks/v3/${appId}/subscriptions`;
                    if (Array.isArray(events) && events.length === 0) {
                        throw new Error(`You must define at least one event`);
                    }
                    for (const event of events) {
                        body = {
                            eventType: event.name,
                            active: true,
                        };
                        if (GenericFunctions_1.propertyEvents.includes(event.name)) {
                            const property = event.property;
                            body.propertyName = property;
                        }
                        await GenericFunctions_1.hubspotApiRequest.call(this, 'POST', endpoint, body);
                    }
                    return true;
                },
                async delete() {
                    const { appId } = this.getCredentials('hubspotDeveloperApi');
                    const { results: subscriptions } = await GenericFunctions_1.hubspotApiRequest.call(this, 'GET', `/webhooks/v3/${appId}/subscriptions`, {});
                    for (const subscription of subscriptions) {
                        await GenericFunctions_1.hubspotApiRequest.call(this, 'DELETE', `/webhooks/v3/${appId}/subscriptions/${subscription.id}`, {});
                    }
                    try {
                        await GenericFunctions_1.hubspotApiRequest.call(this, 'DELETE', `/webhooks/v3/${appId}/settings`, {});
                    }
                    catch (e) {
                        return false;
                    }
                    return true;
                },
            },
        };
    }
    async webhook() {
        const credentials = this.getCredentials('hubspotDeveloperApi');
        if (credentials === undefined) {
            throw new Error('No credentials found!');
        }
        const req = this.getRequestObject();
        const bodyData = req.body;
        const headerData = this.getHeaderData();
        if (headerData['x-hubspot-signature'] === undefined) {
            return {};
        }
        if (credentials.clientSecret !== '') {
            const hash = `${credentials.clientSecret}${JSON.stringify(bodyData)}`;
            const signature = crypto_1.createHash('sha256').update(hash).digest('hex');
            if (signature !== headerData['x-hubspot-signature']) {
                return {};
            }
        }
        for (let i = 0; i < bodyData.length; i++) {
            const subscriptionType = bodyData[i].subscriptionType;
            if (subscriptionType.includes('contact')) {
                bodyData[i].contactId = bodyData[i].objectId;
            }
            if (subscriptionType.includes('company')) {
                bodyData[i].companyId = bodyData[i].objectId;
            }
            if (subscriptionType.includes('deal')) {
                bodyData[i].dealId = bodyData[i].objectId;
            }
            delete bodyData[i].objectId;
        }
        return {
            workflowData: [
                this.helpers.returnJsonArray(bodyData),
            ],
        };
    }
}
exports.HubspotTrigger = HubspotTrigger;
//# sourceMappingURL=HubspotTrigger.node.js.map