"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SendGrid = void 0;
const ListDescription_1 = require("./ListDescription");
const ContactDescription_1 = require("./ContactDescription");
const GenericFunctions_1 = require("./GenericFunctions");
class SendGrid {
    constructor() {
        this.description = {
            displayName: 'SendGrid',
            name: 'sendGrid',
            icon: 'file:sendGrid.svg',
            group: ['transform'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ":" + $parameter["resource"]}}',
            description: 'Consume SendGrid API',
            defaults: {
                name: 'SendGrid',
                color: '#1A82E2',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'sendGridApi',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Contact',
                            value: 'contact',
                        },
                        {
                            name: 'List',
                            value: 'list',
                        },
                    ],
                    default: 'list',
                    required: true,
                    description: 'Resource to consume',
                },
                ...ListDescription_1.listOperations,
                ...ListDescription_1.listFields,
                ...ContactDescription_1.contactOperations,
                ...ContactDescription_1.contactFields,
            ],
        };
        this.methods = {
            loadOptions: {
                async getCustomFields() {
                    const returnData = [];
                    const { custom_fields } = await GenericFunctions_1.sendGridApiRequest.call(this, '/marketing/field_definitions', 'GET', {}, {});
                    if (custom_fields !== undefined) {
                        for (const customField of custom_fields) {
                            returnData.push({
                                name: customField.name,
                                value: customField.id,
                            });
                        }
                    }
                    return returnData;
                },
                async getListIds() {
                    const returnData = [];
                    const lists = await GenericFunctions_1.sendGridApiRequestAllItems.call(this, `/marketing/lists`, 'GET', 'result', {}, {});
                    for (const list of lists) {
                        returnData.push({
                            name: list.name,
                            value: list.id,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const length = items.length;
        const qs = {};
        let responseData;
        const returnData = [];
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        if (resource === 'contact') {
            if (operation === 'getAll') {
                for (let i = 0; i < length; i++) {
                    const returnAll = this.getNodeParameter('returnAll', i);
                    const filters = this.getNodeParameter('filters', i);
                    let endpoint = '/marketing/contacts';
                    let method = 'GET';
                    const body = {};
                    if (filters.query && filters.query !== '') {
                        endpoint = '/marketing/contacts/search';
                        method = 'POST';
                        Object.assign(body, { query: filters.query });
                    }
                    responseData = await GenericFunctions_1.sendGridApiRequestAllItems.call(this, endpoint, method, 'result', body, qs);
                    if (returnAll === false) {
                        const limit = this.getNodeParameter('limit', i);
                        responseData = responseData.splice(0, limit);
                    }
                    returnData.push.apply(returnData, responseData);
                }
            }
            if (operation === 'get') {
                const by = this.getNodeParameter('by', 0);
                let endpoint;
                let method;
                const body = {};
                for (let i = 0; i < length; i++) {
                    if (by === 'id') {
                        method = 'GET';
                        const contactId = this.getNodeParameter('contactId', i);
                        endpoint = `/marketing/contacts/${contactId}`;
                    }
                    else {
                        const email = this.getNodeParameter('email', i);
                        endpoint = '/marketing/contacts/search';
                        method = 'POST';
                        Object.assign(body, { query: `email LIKE '${email}' ` });
                    }
                    responseData = await GenericFunctions_1.sendGridApiRequest.call(this, endpoint, method, body, qs);
                    responseData = responseData.result || responseData;
                    if (Array.isArray(responseData)) {
                        responseData = responseData[0];
                    }
                    returnData.push(responseData);
                }
            }
            if (operation === 'upsert') {
                const contacts = [];
                for (let i = 0; i < length; i++) {
                    const email = this.getNodeParameter('email', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const contact = {
                        email,
                    };
                    if (additionalFields.addressUi) {
                        const addressValues = additionalFields.addressUi.addressValues;
                        const addressLine1 = addressValues.address1;
                        const addressLine2 = addressValues.address2;
                        if (addressLine2) {
                            Object.assign(contact, { address_line_2: addressLine2 });
                        }
                        Object.assign(contact, { address_line_1: addressLine1 });
                    }
                    if (additionalFields.city) {
                        const city = additionalFields.city;
                        Object.assign(contact, { city });
                    }
                    if (additionalFields.country) {
                        const country = additionalFields.country;
                        Object.assign(contact, { country });
                    }
                    if (additionalFields.firstName) {
                        const firstName = additionalFields.firstName;
                        Object.assign(contact, { first_name: firstName });
                    }
                    if (additionalFields.lastName) {
                        const lastName = additionalFields.lastName;
                        Object.assign(contact, { last_name: lastName });
                    }
                    if (additionalFields.postalCode) {
                        const postalCode = additionalFields.postalCode;
                        Object.assign(contact, { postal_code: postalCode });
                    }
                    if (additionalFields.stateProvinceRegion) {
                        const stateProvinceRegion = additionalFields.stateProvinceRegion;
                        Object.assign(contact, { state_province_region: stateProvinceRegion });
                    }
                    if (additionalFields.alternateEmails) {
                        const alternateEmails = additionalFields.alternateEmails.split(',').filter(email => !!email);
                        if (alternateEmails.length !== 0) {
                            Object.assign(contact, { alternate_emails: alternateEmails });
                        }
                    }
                    if (additionalFields.listIdsUi) {
                        const listIdValues = additionalFields.listIdsUi.listIdValues;
                        const listIds = listIdValues.listIds;
                        Object.assign(contact, { list_ids: listIds });
                    }
                    if (additionalFields.customFieldsUi) {
                        const customFields = additionalFields.customFieldsUi.customFieldValues;
                        if (customFields) {
                            const data = customFields.reduce((obj, value) => Object.assign(obj, { [`${value.fieldId}`]: value.fieldValue }), {});
                            Object.assign(contact, { custom_fields: data });
                        }
                    }
                    contacts.push(contact);
                }
                responseData = await GenericFunctions_1.sendGridApiRequest.call(this, '/marketing/contacts', 'PUT', { contacts }, qs);
                returnData.push(responseData);
            }
            if (operation === 'delete') {
                for (let i = 0; i < length; i++) {
                    const deleteAll = this.getNodeParameter('deleteAll', i);
                    if (deleteAll === true) {
                        qs.delete_all_contacts = 'true';
                    }
                    qs.ids = this.getNodeParameter('ids', i).replace(/\s/g, '');
                    responseData = await GenericFunctions_1.sendGridApiRequest.call(this, `/marketing/contacts`, 'DELETE', {}, qs);
                    returnData.push(responseData);
                }
            }
        }
        if (resource === 'list') {
            if (operation === 'getAll') {
                for (let i = 0; i < length; i++) {
                    const returnAll = this.getNodeParameter('returnAll', i);
                    responseData = await GenericFunctions_1.sendGridApiRequestAllItems.call(this, `/marketing/lists`, 'GET', 'result', {}, qs);
                    if (returnAll === false) {
                        const limit = this.getNodeParameter('limit', i);
                        responseData = responseData.splice(0, limit);
                    }
                    returnData.push.apply(returnData, responseData);
                }
            }
            if (operation === 'get') {
                for (let i = 0; i < length; i++) {
                    const listId = this.getNodeParameter('listId', i);
                    qs.contact_sample = this.getNodeParameter('contactSample', i);
                    responseData = await GenericFunctions_1.sendGridApiRequest.call(this, `/marketing/lists/${listId}`, 'GET', {}, qs);
                    returnData.push(responseData);
                }
            }
            if (operation === 'create') {
                for (let i = 0; i < length; i++) {
                    const name = this.getNodeParameter('name', i);
                    responseData = await GenericFunctions_1.sendGridApiRequest.call(this, '/marketing/lists', 'POST', { name }, qs);
                    returnData.push(responseData);
                }
            }
            if (operation === 'delete') {
                for (let i = 0; i < length; i++) {
                    const listId = this.getNodeParameter('listId', i);
                    qs.delete_contacts = this.getNodeParameter('deleteContacts', i);
                    responseData = await GenericFunctions_1.sendGridApiRequest.call(this, `/marketing/lists/${listId}`, 'DELETE', {}, qs);
                    responseData = { success: true };
                    returnData.push(responseData);
                }
            }
            if (operation === 'update') {
                for (let i = 0; i < length; i++) {
                    const name = this.getNodeParameter('name', i);
                    const listId = this.getNodeParameter('listId', i);
                    responseData = await GenericFunctions_1.sendGridApiRequest.call(this, `/marketing/lists/${listId}`, 'PATCH', { name }, qs);
                    returnData.push(responseData);
                }
            }
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.SendGrid = SendGrid;
//# sourceMappingURL=SendGrid.node.js.map