"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimescaleDb = void 0;
const Postgres_node_functions_1 = require("../Postgres/Postgres.node.functions");
const pgPromise = require("pg-promise");
class TimescaleDb {
    constructor() {
        this.description = {
            displayName: 'TimescaleDB',
            name: 'timescaleDb',
            icon: 'file:timescale.svg',
            group: ['input'],
            version: 1,
            description: 'Add and update data in TimescaleDB',
            defaults: {
                name: 'TimescaleDB',
                color: '#fdb515',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'timescaleDb',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    options: [
                        {
                            name: 'Execute Query',
                            value: 'executeQuery',
                            description: 'Execute an SQL query',
                        },
                        {
                            name: 'Insert',
                            value: 'insert',
                            description: 'Insert rows in database',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update rows in database',
                        },
                    ],
                    default: 'insert',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Query',
                    name: 'query',
                    type: 'string',
                    typeOptions: {
                        rows: 5,
                    },
                    displayOptions: {
                        show: {
                            operation: [
                                'executeQuery',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'SELECT id, name FROM product WHERE id < 40',
                    required: true,
                    description: 'The SQL query to execute.',
                },
                {
                    displayName: 'Schema',
                    name: 'schema',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: 'public',
                    required: true,
                    description: 'Name of the schema the table belongs to',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to insert data to.',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'id,name,description',
                    description: 'Comma separated list of the properties which should used as columns for the new rows.',
                },
                {
                    displayName: 'Return Fields',
                    name: 'returnFields',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '*',
                    description: 'Comma separated list of the fields that the operation will return',
                },
                {
                    displayName: 'Schema',
                    name: 'schema',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: 'public',
                    required: true,
                    description: 'Name of the schema the table belongs to',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to update data in',
                },
                {
                    displayName: 'Update Key',
                    name: 'updateKey',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: 'id',
                    required: true,
                    description: 'Name of the property which decides which rows in the database should be updated. Normally that would be "id".',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'name,description',
                    description: 'Comma separated list of the properties which should used as columns for rows to update.',
                },
            ],
        };
    }
    async execute() {
        const credentials = this.getCredentials('timescaleDb');
        if (credentials === undefined) {
            throw new Error('No credentials got returned!');
        }
        const pgp = pgPromise();
        const config = {
            host: credentials.host,
            port: credentials.port,
            database: credentials.database,
            user: credentials.user,
            password: credentials.password,
            ssl: !['disable', undefined].includes(credentials.ssl),
            sslmode: credentials.ssl || 'disable',
        };
        const db = pgp(config);
        let returnItems = [];
        const items = this.getInputData();
        const operation = this.getNodeParameter('operation', 0);
        if (operation === 'executeQuery') {
            const queryResult = await Postgres_node_functions_1.pgQuery(this.getNodeParameter, pgp, db, items);
            returnItems = this.helpers.returnJsonArray(queryResult);
        }
        else if (operation === 'insert') {
            const [insertData, insertItems] = await Postgres_node_functions_1.pgInsert(this.getNodeParameter, pgp, db, items);
            for (let i = 0; i < insertData.length; i++) {
                returnItems.push({
                    json: Object.assign({}, insertData[i]),
                });
            }
        }
        else if (operation === 'update') {
            const updateItems = await Postgres_node_functions_1.pgUpdate(this.getNodeParameter, pgp, db, items);
            returnItems = this.helpers.returnJsonArray(updateItems);
        }
        else {
            await pgp.end();
            throw new Error(`The operation "${operation}" is not supported!`);
        }
        await pgp.end();
        return this.prepareOutputData(returnItems);
    }
}
exports.TimescaleDb = TimescaleDb;
//# sourceMappingURL=TimescaleDb.node.js.map