"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Twist = void 0;
const n8n_core_1 = require("n8n-core");
const GenericFunctions_1 = require("./GenericFunctions");
const ChannelDescription_1 = require("./ChannelDescription");
const MessageConversationDescription_1 = require("./MessageConversationDescription");
const uuid = require("uuid");
class Twist {
    constructor() {
        this.description = {
            displayName: 'Twist',
            name: 'twist',
            icon: 'file:twist.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Twist API',
            defaults: {
                name: 'Twist',
                color: '#316fea',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'twistOAuth2Api',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Channel',
                            value: 'channel',
                        },
                        {
                            name: 'Message Conversation',
                            value: 'messageConversation',
                        },
                    ],
                    default: 'messageConversation',
                    description: 'The resource to operate on.',
                },
                ...ChannelDescription_1.channelOperations,
                ...ChannelDescription_1.channelFields,
                ...MessageConversationDescription_1.messageConversationOperations,
                ...MessageConversationDescription_1.messageConversationFields,
            ],
        };
        this.methods = {
            loadOptions: {
                async getWorkspaces() {
                    const returnData = [];
                    const workspaces = await GenericFunctions_1.twistApiRequest.call(this, 'GET', '/workspaces/get');
                    for (const workspace of workspaces) {
                        returnData.push({
                            name: workspace.name,
                            value: workspace.id,
                        });
                    }
                    return returnData;
                },
                async getConversations() {
                    const returnData = [];
                    const qs = {
                        workspace_id: this.getCurrentNodeParameter('workspaceId'),
                    };
                    const conversations = await GenericFunctions_1.twistApiRequest.call(this, 'GET', '/conversations/get', {}, qs);
                    for (const conversation of conversations) {
                        returnData.push({
                            name: conversation.title || conversation.id,
                            value: conversation.id,
                        });
                    }
                    return returnData;
                },
                async getUsers() {
                    const returnData = [];
                    const qs = {
                        id: this.getCurrentNodeParameter('workspaceId'),
                    };
                    const users = await GenericFunctions_1.twistApiRequest.call(this, 'GET', '/workspaces/get_users', {}, qs);
                    for (const user of users) {
                        returnData.push({
                            name: user.name,
                            value: user.id,
                        });
                    }
                    return returnData;
                },
                async getGroups() {
                    const returnData = [];
                    const qs = {
                        workspace_id: this.getCurrentNodeParameter('workspaceId'),
                    };
                    const groups = await GenericFunctions_1.twistApiRequest.call(this, 'GET', '/groups/get', {}, qs);
                    for (const group of groups) {
                        returnData.push({
                            name: group.name,
                            value: group.id,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < length; i++) {
            if (resource === 'channel') {
                if (operation === 'create') {
                    const workspaceId = this.getNodeParameter('workspaceId', i);
                    const name = this.getNodeParameter('name', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const body = {
                        workspace_id: workspaceId,
                        name,
                    };
                    Object.assign(body, additionalFields);
                    responseData = await GenericFunctions_1.twistApiRequest.call(this, 'POST', '/channels/add', body);
                }
                if (operation === 'get') {
                    const channelId = this.getNodeParameter('channelId', i);
                    qs.id = channelId;
                    responseData = await GenericFunctions_1.twistApiRequest.call(this, 'GET', '/channels/getone', {}, qs);
                }
                if (operation === 'getAll') {
                    const workspaceId = this.getNodeParameter('workspaceId', i);
                    const returnAll = this.getNodeParameter('returnAll', i);
                    const filters = this.getNodeParameter('filters', i);
                    qs.workspace_id = workspaceId;
                    Object.assign(qs, filters);
                    responseData = await GenericFunctions_1.twistApiRequest.call(this, 'GET', '/channels/get', {}, qs);
                    if (!returnAll) {
                        const limit = this.getNodeParameter('limit', i);
                        responseData = responseData.splice(0, limit);
                    }
                }
                if (operation === 'update') {
                    const channelId = this.getNodeParameter('channelId', i);
                    const updateFields = this.getNodeParameter('updateFields', i);
                    const body = {
                        id: channelId,
                    };
                    Object.assign(body, updateFields);
                    responseData = await GenericFunctions_1.twistApiRequest.call(this, 'POST', '/channels/update', body);
                }
            }
            if (resource === 'messageConversation') {
                if (operation === 'create') {
                    const workspaceId = this.getNodeParameter('workspaceId', i);
                    const conversationId = this.getNodeParameter('conversationId', i);
                    const content = this.getNodeParameter('content', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const body = {
                        conversation_id: conversationId,
                        workspace_id: workspaceId,
                        content,
                    };
                    Object.assign(body, additionalFields);
                    if (body.actionsUi) {
                        const actions = body.actionsUi.actionValues;
                        if (actions) {
                            body.actions = actions;
                            delete body.actionsUi;
                        }
                    }
                    if (body.binaryProperties) {
                        const binaryProperties = body.binaryProperties.split(',');
                        const attachments = [];
                        for (const binaryProperty of binaryProperties) {
                            const item = items[i].binary;
                            const binaryData = item[binaryProperty];
                            if (binaryData === undefined) {
                                throw new Error(`No binary data property "${binaryProperty}" does not exists on item!`);
                            }
                            attachments.push(await GenericFunctions_1.twistApiRequest.call(this, 'POST', `/attachments/upload`, {}, {}, {
                                formData: {
                                    file_name: {
                                        value: Buffer.from(binaryData.data, n8n_core_1.BINARY_ENCODING),
                                        options: {
                                            filename: binaryData.fileName,
                                        },
                                    },
                                    attachment_id: uuid(),
                                },
                            }));
                        }
                        body.attachments = attachments;
                    }
                    if (body.direct_mentions) {
                        const direcMentions = [];
                        for (const directMention of body.direct_mentions) {
                            direcMentions.push(`[name](twist-mention://${directMention})`);
                        }
                        body.content = `${direcMentions.join(' ')} ${body.content}`;
                    }
                    responseData = await GenericFunctions_1.twistApiRequest.call(this, 'POST', '/conversation_messages/add', body);
                }
            }
            if (Array.isArray(responseData)) {
                returnData.push.apply(returnData, responseData);
            }
            else {
                returnData.push(responseData);
            }
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.Twist = Twist;
//# sourceMappingURL=Twist.node.js.map