"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeformTrigger = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
class TypeformTrigger {
    constructor() {
        this.description = {
            displayName: 'Typeform Trigger',
            name: 'typeformTrigger',
            icon: 'file:typeform.png',
            group: ['trigger'],
            version: 1,
            subtitle: '=Form ID: {{$parameter["formId"]}}',
            description: 'Starts the workflow on a Typeform form submission.',
            defaults: {
                name: 'Typeform Trigger',
                color: '#404040',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [
                {
                    name: 'typeformApi',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'accessToken',
                            ],
                        },
                    },
                },
                {
                    name: 'typeformOAuth2Api',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'oAuth2',
                            ],
                        },
                    },
                },
            ],
            webhooks: [
                {
                    name: 'default',
                    httpMethod: 'POST',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
            ],
            properties: [
                {
                    displayName: 'Authentication',
                    name: 'authentication',
                    type: 'options',
                    options: [
                        {
                            name: 'Access Token',
                            value: 'accessToken',
                        },
                        {
                            name: 'OAuth2',
                            value: 'oAuth2',
                        },
                    ],
                    default: 'accessToken',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Form',
                    name: 'formId',
                    type: 'options',
                    typeOptions: {
                        loadOptionsMethod: 'getForms',
                    },
                    options: [],
                    default: '',
                    required: true,
                    description: 'Form which should trigger workflow on submission.',
                },
                {
                    displayName: 'Simplify Answers',
                    name: 'simplifyAnswers',
                    type: 'boolean',
                    default: true,
                    description: 'Converts the answers to a key:value pair ("FIELD_TITLE":"USER_ANSER") to be easily processable.',
                },
                {
                    displayName: 'Only Answers',
                    name: 'onlyAnswers',
                    type: 'boolean',
                    default: true,
                    description: 'Returns only the answers of the form and not any of the other data.',
                },
            ],
        };
        this.methods = {
            loadOptions: {
                getForms: GenericFunctions_1.getForms,
            },
        };
        this.webhookMethods = {
            default: {
                async checkExists() {
                    const webhookData = this.getWorkflowStaticData('node');
                    const webhookUrl = this.getNodeWebhookUrl('default');
                    const formId = this.getNodeParameter('formId');
                    const endpoint = `forms/${formId}/webhooks`;
                    const { items } = await GenericFunctions_1.apiRequest.call(this, 'GET', endpoint, {});
                    for (const item of items) {
                        if (item.form_id === formId
                            && item.url === webhookUrl) {
                            webhookData.webhookId = item.tag;
                            return true;
                        }
                    }
                    return false;
                },
                async create() {
                    const webhookUrl = this.getNodeWebhookUrl('default');
                    const formId = this.getNodeParameter('formId');
                    const webhookId = 'n8n-' + Math.random().toString(36).substring(2, 15);
                    const endpoint = `forms/${formId}/webhooks/${webhookId}`;
                    const body = {
                        url: webhookUrl,
                        enabled: true,
                        verify_ssl: true,
                    };
                    await GenericFunctions_1.apiRequest.call(this, 'PUT', endpoint, body);
                    const webhookData = this.getWorkflowStaticData('node');
                    webhookData.webhookId = webhookId;
                    return true;
                },
                async delete() {
                    const formId = this.getNodeParameter('formId');
                    const webhookData = this.getWorkflowStaticData('node');
                    if (webhookData.webhookId !== undefined) {
                        const endpoint = `forms/${formId}/webhooks/${webhookData.webhookId}`;
                        try {
                            const body = {};
                            await GenericFunctions_1.apiRequest.call(this, 'DELETE', endpoint, body);
                        }
                        catch (e) {
                            return false;
                        }
                        delete webhookData.webhookId;
                    }
                    return true;
                },
            },
        };
    }
    async webhook() {
        const bodyData = this.getBodyData();
        const simplifyAnswers = this.getNodeParameter('simplifyAnswers');
        const onlyAnswers = this.getNodeParameter('onlyAnswers');
        if (bodyData.form_response === undefined ||
            bodyData.form_response.definition === undefined ||
            bodyData.form_response.answers === undefined) {
            throw new Error('Expected definition/answers data is missing!');
        }
        const answers = bodyData.form_response.answers;
        const subvalueKeys = [
            'label',
            'labels',
        ];
        if (simplifyAnswers === true) {
            const definition = bodyData.form_response.definition;
            const defintitionsById = {};
            for (const field of definition.fields) {
                defintitionsById[field.id] = field.title.replace(/\{\{/g, '[').replace(/\}\}/g, ']');
            }
            const convertedAnswers = {};
            for (const answer of answers) {
                let value = answer[answer.type];
                if (typeof value === 'object') {
                    for (const key of subvalueKeys) {
                        if (value[key] !== undefined) {
                            value = value[key];
                            break;
                        }
                    }
                }
                convertedAnswers[defintitionsById[answer.field.id]] = value;
            }
            if (onlyAnswers === true) {
                return {
                    workflowData: [
                        this.helpers.returnJsonArray([convertedAnswers]),
                    ],
                };
            }
            else {
                bodyData.form_response.answers = convertedAnswers;
            }
        }
        if (onlyAnswers === true) {
            return {
                workflowData: [
                    this.helpers.returnJsonArray([answers]),
                ],
            };
        }
        else {
            return {
                workflowData: [
                    this.helpers.returnJsonArray([bodyData]),
                ],
            };
        }
    }
}
exports.TypeformTrigger = TypeformTrigger;
//# sourceMappingURL=TypeformTrigger.node.js.map